@@ -51,13 +51,16 @@ import subprocess
 from io import BytesIO
 import streamlit as st
 import pandas as pd
+import plotly.express as px
 
 # from terminal, popen has different paths, so adding the package path manually
 sys.path.append(os.path.dirname(__file__) + '/../')
 import pandas_eda
 
+pd.options.plotting.backend = "plotly"
 
-def download(df, output_name):
+
+def download(df: pd.DataFrame, output_name: str):
     st.write(df)
 
     size_mb = df.memory_usage(deep=True).sum() / 1024 ** 2
@@ -74,6 +77,18 @@ def download(df, output_name):
                            data=output.getvalue(), file_name=f'{output_name}.csv')
 
 
+def add_suffix_to_duplicated_columns(columns):
+    """
+        usage:
+            df.columns = add_suffix_to_duplicated_columns(df.columns)
+    :param columns:
+    :return:
+    """
+    cols = columns.rename('col_name').to_frame().reset_index(drop=True)
+    cols['suffix'] = cols.groupby('col_name').col_name.cumcount().astype(str).radd('_').replace('_0', '')
+    return cols.sum(1)
+
+
 class Main:
     def __init__(self):
 
@@ -83,6 +98,7 @@ class Main:
         self.columns_statistics = None
         self.small_table_to_show = None
         self.mem_size = None
+        self.plot_hist = None
 
         st.set_page_config(layout="wide")
 
@@ -94,8 +110,7 @@ class Main:
          self.tab_config,
          self.tab_help) = st.tabs(['statistics', 'table size', 'frequent values', 'data', 'config', 'help'])
 
-        self.df = pd.read_pickle(sys.argv[1])
-
+        self.read_table()
         self.config()
         self.analyze()
         self.show_data()
@@ -106,9 +121,20 @@ class Main:
 
         print('done reading table')
 
+    def read_table(self):
+        self.df = pd.read_pickle(sys.argv[1])
+        self.df.columns = self.df.columns.astype(str)  # in case one of the columns is integer
+        self.df.columns = add_suffix_to_duplicated_columns(self.df.columns)
+
     def config(self):
         with self.tab_config:
-            self.number_of_most_frequent_values = st.columns(3)[0].slider('number of most frequent values to show', 2, 10, 6)
+            columns = st.columns(3)
+            self.plot_hist = columns[0].checkbox('plot histogram at numeric columns')
+            self.number_of_most_frequent_values = columns[0] \
+                .slider('number of most frequent values to display', 2, 10, 6)
+            columns_to_drop = columns[0].multiselect('select columns to ignore', self.df.columns.tolist())
+            if len(columns_to_drop):
+                self.df = self.df.drop(columns=columns_to_drop)
 
             columns = st.columns([2, 1])
             query = columns[0].text_area('query by table content')
@@ -116,10 +142,6 @@ class Main:
                 st.sidebar.warning('showing filtered data!')
                 self.df = self.df.query(query)
 
-            columns_to_drop = columns[1].multiselect('select columns to ignore', self.df.columns.tolist())
-            if len(columns_to_drop):
-                self.df = self.df.drop(columns=columns_to_drop)
-
     def help(self):
         with self.tab_help:
             st.code(
@@ -164,7 +186,11 @@ class Main:
             st.code(f'{self.mem_size.sum():.3f} MB\n{self.df.shape[0]:,} rows\n{self.df.shape[1]:,} columns')
 
             st.subheader('size of each column:')
-            st.columns(4)[0].table(self.mem_size.to_frame().style.bar(color='#ead9ff').format('{:.3f}'))
+            columns = st.columns([1, 3])
+            columns[0].table(self.mem_size.to_frame().style.bar(color='#ead9ff').format('{:.3f}'))
+            fig = px.pie(self.mem_size.to_frame().reset_index(), names='index', values='MB', hole=0.4)
+            fig.update_traces(texttemplate="%{label}:<br>%{value:,.3f}MB<br>%{percent:.0%}",)
+            columns[1].plotly_chart(fig)
 
     def show_frequent_values(self):
         with self.tab_frequent_values:
@@ -173,6 +199,7 @@ class Main:
             frequent.value = frequent.value.astype(str)  # streamlit issue
             download(frequent, 'frequent_values')
 
+        # now the sidebar
         st.sidebar.header('frequent values per column:')
         freq = self.eda.get_frequent_values()
         for col in freq.col.unique():
@@ -182,9 +209,20 @@ class Main:
             st.sidebar.info(f'{col} [{single_col_statistics.uniques:,} uniques, {single_col_statistics.nans:,} nans]')
             show = freq.query(f'col=="{col}"').set_index('value')
             for index, row in show.iterrows():
-                st.sidebar.write(f'{index}: [#{row.counts:,}]')
+                st.sidebar.write(f'{index}: [#{row.counts:,}, {row.counts/self.df.shape[0]:.2%}]')
                 st.sidebar.progress(row.percentages)
 
+            if self.plot_hist:
+                if pd.api.types.is_numeric_dtype(self.df[col]) and single_col_statistics.uniques > show.shape[0]:
+                    bins = st.sidebar.slider('number of bins',
+                                             0,
+                                             min(self.df[col].nunique(), 300),
+                                             0,
+                                             key=f'nbins {col}')
+                    bins = bins if bins > 1 else None
+                    st.sidebar.plotly_chart(self.df.plot.hist(x=col, height=300, nbins=bins),
+                                            use_container_width=True)
+
 
 if __name__ == '__main__':
     if st._is_running_with_streamlit:
